#!/usr/bin/env python3
"""
SMTP Server Testing Tool
Parameters:
    - IP: SMTP server IP address
    - port: SMTP server port
Additional parameters in comments:
    - username: SMTP username
    - password: SMTP password
    - test_type: Type of test to perform
"""
import smtplib
import dns.resolver
import socket
import ssl
import argparse
import logging
from datetime import datetime
import json
from pathlib import Path
import threading
import queue
import time
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart

class SMTPTester:
    def __init__(self, server_ip, port=25, username=None, password=None,
                use_ssl=False, timeout=10):
        self.server_ip = server_ip
        self.port = port
        self.username = username
        self.password = password
        self.use_ssl = use_ssl
        self.timeout = timeout
        
        self.results_queue = queue.Queue()
        self.setup_logging()
        
    def setup_logging(self):
        """Set up logging configuration"""
        log_dir = Path("smtp_logs")
        log_dir.mkdir(exist_ok=True)
        
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s',
            handlers=[
                logging.FileHandler(log_dir / "smtp_test.log"),
                logging.StreamHandler()
            ]
        )
        
    def check_dns(self):
        """Check DNS records for SMTP server"""
        results = {'dns_checks': []}
        
        try:
            # Get IP from hostname
            try:
                ip = socket.gethostbyname(self.server_ip)
                results['dns_checks'].append({
                    'check': 'IP resolution',
                    'status': 'success',
                    'result': ip
                })
            except socket.gaierror as e:
                results['dns_checks'].append({
                    'check': 'IP resolution',
                    'status': 'error',
                    'error': str(e)
                })
                
            # Check MX records
            try:
                mx_records = dns.resolver.resolve(self.server_ip, 'MX')
                results['dns_checks'].append({
                    'check': 'MX records',
                    'status': 'success',
                    'result': [str(mx.exchange) for mx in mx_records]
                })
            except Exception as e:
                results['dns_checks'].append({
                    'check': 'MX records',
                    'status': 'error',
                    'error': str(e)
                })
                
            # Check SPF record
            try:
                txt_records = dns.resolver.resolve(self.server_ip, 'TXT')
                spf_records = [str(record) for record in txt_records if 'spf' in str(record).lower()]
                results['dns_checks'].append({
                    'check': 'SPF record',
                    'status': 'success' if spf_records else 'warning',
                    'result': spf_records if spf_records else 'No SPF record found'
                })
            except Exception as e:
                results['dns_checks'].append({
                    'check': 'SPF record',
                    'status': 'error',
                    'error': str(e)
                })
                
            # Check DMARC record
            try:
                dmarc_records = dns.resolver.resolve(f"_dmarc.{self.server_ip}", 'TXT')
                results['dns_checks'].append({
                    'check': 'DMARC record',
                    'status': 'success',
                    'result': [str(record) for record in dmarc_records]
                })
            except Exception as e:
                results['dns_checks'].append({
                    'check': 'DMARC record',
                    'status': 'warning',
                    'error': 'No DMARC record found'
                })
                
        except Exception as e:
            logging.error(f"Error during DNS checks: {str(e)}")
            
        return results
        
    def test_connection(self):
        """Test basic SMTP connection"""
        results = {'connection_test': {}}
        
        try:
            start_time = time.time()
            
            # Create appropriate SMTP connection
            if self.use_ssl:
                smtp = smtplib.SMTP_SSL(self.server_ip, self.port,
                                    timeout=self.timeout)
            else:
                smtp = smtplib.SMTP(self.server_ip, self.port,
                                timeout=self.timeout)
                
            # Try STARTTLS if not using SSL
            if not self.use_ssl:
                try:
                    smtp.starttls()
                    results['connection_test']['starttls'] = 'supported'
                except:
                    results['connection_test']['starttls'] = 'not supported'
                    
            # Get server banner
            code, banner = smtp.ehlo()
            
            results['connection_test'].update({
                'status': 'success',
                'response_time': time.time() - start_time,
                'banner': banner.decode() if isinstance(banner, bytes) else str(banner),
                'features': []
            })
            
            # Get supported features
            for feature in banner.decode().split('\n')[1:]:
                results['connection_test']['features'].append(feature.strip())
                
            smtp.quit()
            
        except Exception as e:
            results['connection_test'].update({
                'status': 'error',
                'error': str(e)
            })
            
        return results
        
    def test_authentication(self):
        """Test SMTP authentication"""
        results = {'auth_test': {}}
        
        if not (self.username and self.password):
            results['auth_test']['status'] = 'skipped'
            return results
            
        try:
            # Create connection
            if self.use_ssl:
                smtp = smtplib.SMTP_SSL(self.server_ip, self.port,
                                    timeout=self.timeout)
            else:
                smtp = smtplib.SMTP(self.server_ip, self.port,
                                timeout=self.timeout)
                smtp.starttls()
                
            # Try authentication
            start_time = time.time()
            smtp.login(self.username, self.password)
            
            results['auth_test'].update({
                'status': 'success',
                'response_time': time.time() - start_time
            })
            
            smtp.quit()
            
        except Exception as e:
            results['auth_test'].update({
                'status': 'error',
                'error': str(e)
            })
            
        return results
        
    def test_send_email(self, from_addr, to_addr):
        """Test sending email"""
        results = {'send_test': {}}
        
        try:
            # Create test message
            msg = MIMEMultipart()
            msg['From'] = from_addr
            msg['To'] = to_addr
            msg['Subject'] = f"SMTP Test {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}"
            
            body = "This is a test email sent by SMTP Tester tool."
            msg.attach(MIMEText(body, 'plain'))
            
            # Create connection
            if self.use_ssl:
                smtp = smtplib.SMTP_SSL(self.server_ip, self.port,
                                    timeout=self.timeout)
            else:
                smtp = smtplib.SMTP(self.server_ip, self.port,
                                timeout=self.timeout)
                smtp.starttls()
                
            # Authenticate if credentials provided
            if self.username and self.password:
                smtp.login(self.username, self.password)
                
            # Send email
            start_time = time.time()
            smtp.send_message(msg)
            
            results['send_test'].update({
                'status': 'success',
                'response_time': time.time() - start_time
            })
            
            smtp.quit()
            
        except Exception as e:
            results['send_test'].update({
                'status': 'error',
                'error': str(e)
            })
            
        return results
        
    def test_smtp_commands(self):
        """Test various SMTP commands"""
        results = {'command_tests': []}
        
        try:
            # Create connection
            if self.use_ssl:
                smtp = smtplib.SMTP_SSL(self.server_ip, self.port,
                                    timeout=self.timeout)
            else:
                smtp = smtplib.SMTP(self.server_ip, self.port,
                                timeout=self.timeout)
                smtp.starttls()
                
            # Test EHLO
            try:
                code, response = smtp.ehlo()
                results['command_tests'].append({
                    'command': 'EHLO',
                    'status': 'success',
                    'response_code': code,
                    'response': response.decode() if isinstance(response, bytes) else str(response)
                })
            except Exception as e:
                results['command_tests'].append({
                    'command': 'EHLO',
                    'status': 'error',
                    'error': str(e)
                })
                
            # Test HELP
            try:
                code, response = smtp.help()
                results['command_tests'].append({
                    'command': 'HELP',
                    'status': 'success',
                    'response_code': code,
                    'response': response.decode() if isinstance(response, bytes) else str(response)
                })
            except Exception as e:
                results['command_tests'].append({
                    'command': 'HELP',
                    'status': 'error',
                    'error': str(e)
                })
                
            smtp.quit()
            
        except Exception as e:
            logging.error(f"Error during command tests: {str(e)}")
            
        return results
        
    def run_tests(self, test_types=None):
        """Run all specified tests"""
        if test_types is None:
            test_types = ['dns', 'connection', 'auth', 'commands']
            
        results = {
            'timestamp': datetime.now().isoformat(),
            'server': self.server_ip,
            'port': self.port,
            'ssl': self.use_ssl
        }
        
        if 'dns' in test_types:
            results.update(self.check_dns())
            
        if 'connection' in test_types:
            results.update(self.test_connection())
            
        if 'auth' in test_types:
            results.update(self.test_authentication())
            
        if 'commands' in test_types:
            results.update(self.test_smtp_commands())
            
        # Save results
        self.save_results(results)
        
    def save_results(self, results):
        """Save test results to file"""
        report_file = Path("smtp_logs") / f"report_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json"
        with open(report_file, 'w') as f:
            json.dump(results, f, indent=2)
            
        logging.info(f"Test results saved to {report_file}")
        
        # Print summary
        self.print_summary(results)
        
    def print_summary(self, results):
        """Print test results summary"""
        print("\nSMTP Server Test Summary")
        print("=" * 50)
        print(f"Server: {self.server_ip}:{self.port}")
        print(f"SSL/TLS: {'Yes' if self.use_ssl else 'No'}")
        print("-" * 50)
        
        # DNS checks
        if 'dns_checks' in results:
            print("\nDNS Checks:")
            for check in results['dns_checks']:
                status = "✓" if check['status'] == 'success' else "✗"
                print(f"{status} {check['check']}")
                
        # Connection test
        if 'connection_test' in results:
            print("\nConnection Test:")
            if results['connection_test'].get('status') == 'success':
                print("✓ Connection successful")
                print(f"Response time: {results['connection_test']['response_time']:.3f}s")
            else:
                print("✗ Connection failed")
                
        # Authentication test
        if 'auth_test' in results:
            print("\nAuthentication Test:")
            if results['auth_test'].get('status') == 'success':
                print("✓ Authentication successful")
            elif results['auth_test'].get('status') == 'skipped':
                print("- Authentication skipped (no credentials)")
            else:
                print("✗ Authentication failed")
                
        # Command tests
        if 'command_tests' in results:
            print("\nCommand Tests:")
            for test in results['command_tests']:
                status = "✓" if test['status'] == 'success' else "✗"
                print(f"{status} {test['command']}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="SMTP Server Testing Tool")
    parser.add_argument("server", help="SMTP server IP or hostname")
    parser.add_argument("--port", type=int, default=25,
                      help="SMTP server port")
    parser.add_argument("--username", help="SMTP username")
    parser.add_argument("--password", help="SMTP password")
    parser.add_argument("--ssl", action="store_true",
                      help="Use SSL/TLS connection")
    parser.add_argument("--timeout", type=int, default=10,
                      help="Connection timeout in seconds")
    parser.add_argument("--tests", choices=['dns', 'connection', 'auth', 'commands', 'all'],
                      default='all', help="Tests to run")
    args = parser.parse_args()
    
    tester = SMTPTester(
        args.server, args.port, args.username, args.password,
        args.ssl, args.timeout
    )
    
    test_types = ['dns', 'connection', 'auth', 'commands'] if args.tests == 'all' else [args.tests]
    tester.run_tests(test_types)
